﻿// Wheel of Trinkets - Actionscript 3 Version by Jim Bumgardner  9/6/2008

package 
{
	import flash.events.Event;
	import flash.display.Sprite;
	import flash.display.DisplayObject;
	import flash.display.GradientType;
	import flash.events.MouseEvent;
	import flash.geom.Matrix;

	public class Trinket_mc extends Sprite
	{
		static public const kNumberGuys = 50;     // Number of balls
		
		static private const kMinRadius = 8;       // Radius of balls
		static private const kMaxRadius = 32;      // Radius of balls
		static private const kCollisionDamp = .6;  // Helps reduce jitter from collisions
		
		static public var kDamp = .9;             // Decreasing this makes the balls act like they are in thick oil
		static public var kGravWeight = 1; 	       // Controls how strong the force towards the center is
		static public var kSpin = 1;
		static public var kSpinR = kSpin*Math.PI/180;

		static private const kRadiansToDegrees = 180/Math.PI;
		static private const kDegreesToRadians = Math.PI/180;
	
		static private var SW,SH;
		static private var guys = [];

		static private var gTx = 0; // direction of gravity
		static private var gTy = 1;
		
		private var dragging:Boolean = false;
		private var vx,vy,lx,ly;
		public var radius;
		static var objcell_mc;
	  
		public static function InitBalls(itsPar:DisplayObject)
		{
			trace("got init");
			objcell_mc = new Sprite();
			SW = itsPar.stage.stageWidth;
			SH = itsPar.stage.stageWidth;
			
			objcell_mc.x = SW/2;
			objcell_mc.y = SH/2;
			itsPar.addChild(objcell_mc);
			
			for (var j = 0; j < kNumberGuys; ++j) {
			  var mc = new Trinket_mc(itsPar);
			  objcell_mc.addChild(mc);
			}
			drawSphere(objcell_mc.graphics, 0x777777, SW/2, .3);
			objcell_mc.addEventListener(Event.ENTER_FRAME, Trinket_mc.objspin);
		}
		
		public function Trinket_mc(itsPar:DisplayObject)
		{
			buttonMode = true; // use hand cursors
			guys.push(this);
			x = Math.random()*SW-SW/2;
			y = Math.random()*SH-SH/2;
			x *= .5;
			y *= .5;
			vx = vy = 0;
			radius = kMinRadius + Math.random()*(kMaxRadius - kMinRadius);
			drawSphere(graphics, getCandyColor(guys.length/(kNumberGuys-1)), radius, 1);
			addEventListener(Event.ENTER_FRAME, this.clump_about);
			addEventListener(MouseEvent.MOUSE_DOWN, this.pressBall);
			addEventListener(MouseEvent.MOUSE_UP, this.releaseBall);
		}
	
		function pressBall(evt)
		{
		  dragging = true; 
		  lx = parent.root.mouseX;
		  ly = parent.root.mouseY; 
		  startDrag(true);
		}
	
		function releaseBall(evt)
		{
		  dragging = false;
		  stopDrag();
		}
	
		private static function tintColor(a,v,mix)
		{
		  var r1 = a >> 16;
		  var g1 = (a >> 8) & 0xFF;
		  var b1 = a & 0xFF;
		  r1 = r1*(1-mix)+v*mix;
		  g1 = g1*(1-mix)+v*mix;
		  b1 = b1*(1-mix)+v*mix;
		  return (r1 << 16) | (g1 << 8) | b1;
		}
		
		private static function getRainbowColor(r) // r is 0 - 1  - returns a bright color
		{
		  var a = r*Math.PI*2;
		  var r = 128+Math.cos(a)*127;
		  var b = 128+Math.cos(a+2*Math.PI/3)*127;
		  var g = 128+Math.cos(a+4*Math.PI/3)*127;
		  return (r << 16) | (g << 8) | b;
		}
		
		private static function getCandyColor(r)
		{
		  var v = 128+Math.floor(Math.random()*127);
		  return tintColor(getRainbowColor(r),v,.5);
		}
	
		private static function drawSphere(graf, clr, r, alfa)
		{
			var alphas = [alfa,alfa,alfa];
			var ratios = [0x00,0x44,0xFF];
			var colors = [0xFFFFFF,clr,tintColor(clr,0,.5)];
			
			//      var matrix = { matrixType:"box", x:-r, y:-r*2, w:r*3, h:r*3, r: 0 }
			var matrix = new Matrix();
			matrix.createGradientBox(r*3, r*3, 0, -r, -r*2);
			
			graf.clear();
			// graphics.beginFill(clr, 100);
			graf.beginGradientFill(GradientType.RADIAL,colors,alphas,ratios,matrix);
			graf.drawCircle(0,0,r);
			graf.endFill();
		}
		
		public function myHitTest(mc:Trinket_mc)
		{
			// old style test in comments: for rectangle intersection...
			return mc.x-mc.width/2 < x+width/2 && mc.x+mc.width/2 >= x-width/2 &&
			       mc.y-mc.height/2 < y+height/2 && mc.y+mc.height/2 >= y-width/2;
		}
		
		function clump_about(evt)
		{
			if (dragging) {
				vx = parent.root.mouseX - lx;
				vy = parent.root.mouseY - ly;
				lx = parent.root.mouseX;
				ly = parent.root.mouseY;
				return;
			}
			// friction
			vx *= kDamp;
			vy *= kDamp;

			// gravity
			this.vx += gTx*kGravWeight;
			this.vy += gTy*kGravWeight;

			// Collision detection with balls (Avoidance)
			var fx = 0;
			var fy = 0;
			var dx,dy,dist,scale;
			var dmax;
			
			for (var i = guys.length-1; i >= 0; --i) 
			{
				if (guys[i] != this && myHitTest(guys[i])) 
				{
					dx = (x+vx) - guys[i].x;
					dy = (y+vy) - guys[i].y;
					dist = Math.sqrt(dx*dx+dy*dy);
					maxdist = this.radius+guys[i].radius;
					dmax = maxdist - dist;
					if (dmax > 0) {
						var mag1 = dmax*kCollisionDamp/maxdist;
						// mag1 *= mag1;
						fx += dx*mag1;
						fy += dy*mag1;
					}
				}
			}
			vx += fx;
			vy += fy;
			

			// Rim Collisions
			dx = 0-(x+vx);
			dy = 0-(y+vy);
			dist = Math.sqrt(dx*dx+dy*dy);
			dmax = dist - (SW/2-radius);
			if (dmax > 0) {
				vx += dx*dmax/dist;
				vy += dy*dmax/dist;
			}
      
			this.x += vx;
			this.y += vy;
		} // end of function

		
		public static function objspin(evt)
		{
		  for (var i = guys.length-1; i >= 0; --i) 
		  {
			var guy = guys[i];
			var dist = Math.sqrt(guy.x*guy.x + guy.y*guy.y);
			var ang = Math.atan2(guy.y, guy.x);
			guy.x = Math.cos(ang+kSpinR)*dist;
			guy.y = Math.sin(ang+kSpinR)*dist;
		  }
		}
	} // end of clas

} // end of package
